// Definicija klase za relativne datoteke (RFile).

#include <fstream>
#include <string>
using namespace std;

class RFile : fstream {
public:
  class Greska { // Ugnezdjena klasa za predstavljanje gresaka.
  public:
    enum Gre { OK ,      // Sifre gresaka:
               DUZ,      //   neispravna duzina zapisa,
               BRO,      //   neispravan redni broj zapisa,
               VEL,      //   datoteka nije ceo broj zapisa,
               ZAT,      //   datoteka nije otvorena,
               IMA,      //   datoteka vec postoji,
               NEMA,     //   datoteka ne postoji,
               OTV,      //   nije uspelo otvaranje,
               POZ,      //   nije uspelo pozicioniranje,
               PIS,      //   nije uspelo upisivanje,
               CIT,      //   nije uspelo citanje,
               ZAP       //   zapis ne postoji.
             };
  private:
    Gre gre;                    // Sifra greske.
    static const string por[];  // Tekstualni opisi gresaka.
  public:
    explicit Greska(Gre g) { gre = g; }
    friend ostream& operator<<(ostream& it, const Greska& g);
  };

private:
  long duz_zap, bro_zap, tek_zap, tek_poz;  // bro_zap==-1 - nije otvorena.
  void otvori(const string& ime, long duz=0);  // Otvaranje datoteke (duz>0
                                               //   - stvaranje nove).
public:
  RFile(): fstream() { bro_zap = -1; }   // Inic. bez otvaranja datoteke.
  explicit RFile(const string& ime, long duz=0)   // Inic. sa otvaranjem
    : fstream() { otvori(ime, duz); }             //   datoteke.
  ~RFile() { close(); }                           // Unistavanje objekta.
  RFile& open(const string& ime, long duz=0)      // Otvaranje datoteke.
    { otvori(ime, duz); return *this; }
  int open() { return bro_zap >= 0; }       // Da li je datoteka otvorena?
  RFile& write(const void* niz, long zap=0);// Pisanje zap(zap==0 - sekv).
  RFile& read(void* niz, long zap=0);       // Citanje zap(zap==0 - sekv).
  RFile& close();                           // Zatvaranje datoteke.
  long recn() {return bro_zap<0 ? -1 : bro_zap;} // Broj zapisa u datoteci.
  long recl() {return bro_zap<0 ? -1 : duz_zap;} // Duzina zapisa.
};

